import pandas as pd
import click
import logging
from IVF_extremes.IVF_func import *
import os



@click.command()
@click.option('-i','--input_file', required=True,help='Path to the input file.')
@click.option('-o','--out_path', required = False, default='out_IVF', show_default=True, help='Path to the input file.')
@click.option('-e','--estimate-baseline', is_flag=True, show_default=True, help='Estimate IVF baseline rates from input dataset')
@click.option('--om_rate', default=0.8, show_default=True, help='Baseline Oocyte Maturation Rate')
@click.option('--of_rate', default=0.8, show_default=True, help='Baseline Oocyte Fertilization Rate')
@click.option('--blast_rate', default=0.6, show_default=True, help='Baseline Blastocyst Production Rate')
@click.option('--pvalue_th', help='Pvalue threshold for outliers')
def CLI_batch(input_file, out_path='output_IVF_extremes', estimate_baseline=False, om_rate=0.8, of_rate=0.8, blast_rate=0.6, pvalue_th=None):
    batch_check(input_file, out_path, estimate_baseline, om_rate, of_rate, blast_rate, pvalue_th)

def batch_check(input_file, out_path='output_IVF_extremes', estimate_baseline=False, om_rate=0.8, of_rate=0.8, blast_rate=0.6, pvalue_th=None):
    """IVF-extremes

    Analyze IVF clinical data and identify phenotypic extremes"""

    os.makedirs(out_path, exist_ok=True)
    logging.basicConfig(filename=os.path.join(out_path,'IVF_extremes.log'), encoding='utf-8', level=logging.DEBUG,filemode='w',format='%(asctime)s %(levelname)s %(message)s')
    logging.getLogger().addHandler(logging.StreamHandler())

    # 1 parse input data and aggregate by patient ID
    logging.info('IVF_extremes: analysis started')
    logging.info('Parsing input data')
    df_IVF = pd.read_csv(input_file)
    df_patients = df_IVF.groupby(['Patient_ID', 'Age'])[['COC', 'MII', 'Fertilized', 'Blastocysts']].sum().reset_index()
    # 2 estimate baseline data if not provided
    if estimate_baseline:
        logging.info('Estimating baseline from input dataset')
        OM_rate, OF_rate, B_rate_age = estimate_baseline_rate(df_patients)
    else:
        logging.info('Fixing baseline with custom values')
        OM_rate, OF_rate = (om_rate,of_rate)
        B_rate_age = {0:blast_rate,100:blast_rate}
    # 3 compute pvalues
    logging.info('Computing pvalues')
    df_pvalues = compute_pvalues(df_patients, OM_rate, OF_rate, B_rate_age)
    # 4 identify outliers
    logging.info('Identifying outliers')
    df_outliers = find_IVF_outliers(df_pvalues, pvalue_th)
    # 5 output results
    logging.info('Exporting results to {} folder'.format(out_path))
    df_pvalues.to_csv(os.path.join(out_path, 'IVF_pvalues.csv'), index=False)
    df_outliers.to_csv(os.path.join(out_path, 'IVF_outliers.csv'), index=False)

    return df_outliers


if __name__ == '__batch_check__':
    batch_check()





import os
import unittest
import IVF_extremes

out_test_folder = 'out_test'
data_path = '../data/Sample_IVF_data.csv'

class TestIVF(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        for f in os.listdir(out_test_folder):
            os.remove(os.path.join(out_test_folder,f))
        os.rmdir(out_test_folder)

    def test_IVF_nobaseline(self):
        df_outliers = IVF_extremes.batch_check(data_path,out_path=out_test_folder)
        outliers = list(df_outliers.Patient_ID.unique())
        expected_outlier = [9,15,16,29,40,44,47,52,55,67,76,79,86]
        self.assertEqual(outliers,  expected_outlier)

    def test_IVF_baseline(self):
        df_outliers = IVF_extremes.batch_check(data_path,out_path=out_test_folder,estimate_baseline=True)
        outliers = list(df_outliers.Patient_ID.unique())
        expected_outlier = [9, 13, 15, 51, 53, 55, 67, 76, 79, 86]
        self.assertEqual(outliers,  expected_outlier)

    def test_upper(self):
        self.assertEqual('foo'.upper(), 'FOO')

    def test_isupper(self):
        self.assertTrue('FOO'.isupper())
        self.assertFalse('Foo'.isupper())

    def test_split(self):
        s = 'hello world'
        self.assertEqual(s.split(), ['hello', 'world'])
        # check that s.split fails when the separator is not a string
        with self.assertRaises(TypeError):
            s.split(2)

def suite():
    suite = unittest.TestSuite()
    suite.addTest(TestIVF('test_IVF_baseline'))
    suite.addTest(TestIVF('test_IVF_nobaseline'))
    return suite

if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    runner.run(suite())



import click
import logging
import pandas as pd
from IVF_extremes.IVF_func import *
import os

resources_dir = os.path.join(os.path.dirname(__file__), 'resources')


@click.command()
@click.option('--coc', required=True, type=int, help='number of cocs')
@click.option('--mii', required=True, type=int, help='number of mature oocytes')
@click.option('--fert', required=True, type=int, help='number of fertilized oocytes')
@click.option('--blast', required=True, type=int, help='Number of developed blastocysts')
@click.option('--age', required=True, type=int, help='Age of the woman')
@click.option('--pvalue_th', default=0.05, show_default=True, help='Pvalue threshold for outliers')
def CLI_single_check(coc, mii, fert, blast, age, pvalue_th):
    single_check(coc, mii, fert, blast, age, pvalue_th)

def single_check(num_cocs, num_mature_oocytes, num_fertlized_oocytes, num_blast, age, pvalue_th=0.05):
    """IVF-extremes

    Analyze IVF clinical data and identify phenotypic extremes"""

    logging.basicConfig(filename='IVF_extremes.log', encoding='utf-8', level=logging.DEBUG,filemode='w',format='%(asctime)s %(levelname)s %(message)s')
    logging.getLogger().addHandler(logging.StreamHandler())

    # 1 parse input data and aggregate by patient ID
    logging.info('\n*** IVF_extremes: analysis started\n')
    baseline_rate = pd.read_csv(os.path.join(resources_dir,'baseline_rates.csv'),index_col='Female Age')
    B_rate = baseline_rate['BR'].iloc[age]
    OM_rate = baseline_rate['OMR'].mean()
    OF_rate = baseline_rate['OFR'].mean()
    logging.info('   Baseline Maturation Rate={:.2f},\n   Baseline Fertilization Rate={:.2f},'
                 '\n   Baseline Blastulation Rate={:.2f}\n'.format(OM_rate,OF_rate,B_rate))

    logging.info('*** Computing pvalues...')
    p_LMR, p_LFR, p_PDA = IVF_binomial_tests(num_cocs, num_mature_oocytes, num_fertlized_oocytes, num_blast, OM_rate, OF_rate, B_rate)
    logging.info('   pvalue Low Maturation Rate={:.2E},\n   pvalue Low Fertilization Rate={:.2E},'
                 '\n   pvalue Preimplantation Development Arrest={:.2E}\n'.format(p_LMR,p_LFR,p_PDA))

    # 4 identify outliers
    logging.info('*** Identifying outliers...')
    outlier_LMR = p_LMR < pvalue_th
    outlier_LFR = p_LFR < pvalue_th
    outlier_PDA = p_PDA < pvalue_th
    if (outlier_LFR)|(outlier_LMR)|(outlier_PDA):
        logging.info('   IVF_extremes outcome: Abnormal IVF values')
        logging.info('   Outlier LMR: {}\n   Outlier LFR: {}\n   Outlier PDA: {}\n'.format(outlier_LMR,outlier_LFR,outlier_PDA))
    else:
        logging.info('   IVF_extremes outcome: Normal IVF values\n')

    return


if __name__ == '__single_check__':
    single_check()





from scipy import stats
from scipy.interpolate import interp1d
import logging

def estimate_baseline_rate(df_patients):
    "Estimate baseline rate from input dataframe"
    OM_rate = (df_patients['MII'] / df_patients['COC']).mean()
    OF_rate = (df_patients['Fertilized'] / df_patients['MII']).mean()
    fert_age = df_patients.groupby('Age')['Fertilized'].sum()
    bla_age = df_patients.groupby('Age')['Blastocysts'].sum()
    B_rate_age = ( bla_age / fert_age ).to_dict()
    return OM_rate, OF_rate, B_rate_age


def compute_pvalues(df_protocol, OM_rate, OF_rate, B_rate_age):
    "Perform binomial hypothesis tests to compare individual IVF outcomes with baseline rates"
    for i in range(len(df_protocol)):
        Patient_ID = df_protocol.loc[i, 'Patient_ID']
        n_coc = df_protocol.loc[i, 'COC']
        n_mII = df_protocol.loc[i, 'MII']
        n_fert = df_protocol.loc[i, 'Fertilized']
        n_bla = df_protocol.loc[i, 'Blastocysts']
        age = df_protocol.loc[i, 'Age']
        x = list(B_rate_age.keys())
        y = list(B_rate_age.values())
        func_age = interp1d(x, y)
        B_rate = func_age(age)
        p_LMR, p_LFR, p_PDA = IVF_binomial_tests(n_coc, n_mII, n_fert, n_bla, OM_rate, OF_rate, B_rate, Patient_ID)
        df_protocol.loc[i, 'pvalue_LFR'] = p_LFR
        df_protocol.loc[i, 'pvalue_LMR'] = p_LMR
        df_protocol.loc[i, 'pvalue_PDA'] = p_PDA
    return df_protocol


def IVF_binomial_tests(coc, mII, fert, bla, OM_rate, OF_rate, B_rate, Patient_ID=''):
    "Binomial tests to estimate probabilities of IVF outcome"

    try:
        if (coc < mII) | (mII < fert) | (fert < bla):
            logging.warning(
                "Inconsistent IVF values for Patient_ID={}: coc={}, mII={}, fert={}, bla={}".format(Patient_ID, coc,
                                                                                                    mII, fert,bla))
            raise ValueError
        p_LMR = stats.binom_test(x=mII, n=coc, p=OM_rate, alternative='less')
        p_LFR = stats.binom_test(x=fert, n=mII, p=OF_rate, alternative='less')
        p_PDA = stats.binom_test(x=bla, n=fert, p=B_rate, alternative='less')
    except:
        p_LMR, p_LFR, p_PDA = None, None, None

    return p_LMR, p_LFR, p_PDA


def find_IVF_outliers(df_protocol, pvalue_th):
    "Classify individuals as outliers if the pvalue from the binomial test is lower than threshold"
    if pvalue_th is None:
        pvalue_th = 0.05 / df_protocol.shape[1]  # bonferroni correction
    df_protocol['LMR_outlier'] = df_protocol['pvalue_LMR'] < pvalue_th
    df_protocol['LFR_outlier'] = df_protocol['pvalue_LFR'] < pvalue_th
    df_protocol['PDA_outlier'] = df_protocol['pvalue_PDA'] < pvalue_th
    df_outliers = df_protocol[
        (df_protocol['LMR_outlier']) | (df_protocol['LFR_outlier']) | (df_protocol['PDA_outlier'])]
    logging.info('Found the following outliers:\n\nEndophenotype\tNumber of cases\n{}\n'.format(df_outliers[['LMR_outlier','LFR_outlier','PDA_outlier']].sum()))
    return df_outliers[['Patient_ID', 'LMR_outlier', 'LFR_outlier', 'PDA_outlier']]
